#ifndef EDITLIST_H
#define EDITLIST_H

/*! 
  \file editlist.h

  \brief Handling different edit list formats

  \author R. Enders

  $Revision: 1.11 $
  $Id: editlist.h,v 1.11 2006/01/06 12:17:58 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include editlist.h
*/

/*
  change the interface in a way, that qt stays out -
  do fileselection in gopdit.cpp
*/
#include "config.h"
#include <vector>
#include <string>
#include "parameter.h"
#include "pesmgr.h"

class Gopdit;
class GopditUi;
class PesMgr;


class ClipPointSearcher
{
public:
    ClipPointSearcher(PesMgr* ppesmgr)
	: pesmgr(ppesmgr)
    {}
    ClipPoint getLastClip() const;
protected:
    PesMgr*   pesmgr;

    unsigned   offset2ms(off_t offset) const;
    ClipPoint ms2offset(unsigned ms) const;
};

class AbstractEditListMgr : public ClipPointSearcher
{
public:
    AbstractEditListMgr(FILE* pfp, Gopdit& gd, GopditUi& pui);
    virtual ~AbstractEditListMgr()
    {}
    virtual Mpeg2ClipList importlist() = 0;
    virtual void exportlist(const Mpeg2ClipList& clip_list) = 0;
protected:
    FILE*     fp;
    Gopdit&   gopdit;
    GopditUi& ui;
};

class GopditEditListMgr
{
public:
    GopditEditListMgr(GopditUi& pui, Gopdit& gd)
	: ui(pui)
	, gopdit(gd)
    {}
    Mpeg2ClipList importList(std::string filename, std::string filter);
    void exportList(std::string filename, std::string filter,
		    const Mpeg2ClipList& cut_list);
    AbstractEditListMgr* genEditListMgr(std::string filter,  FILE* fp);
protected:
    GopditUi&  ui;
    Gopdit&    gopdit;
};

extern FileDialogParameter export_file;
extern FileDialogParameter import_file;


#endif /* EDITLIST_H */
