/*! 
  \file gopdit.cpp
  \brief

  \author R. Enders

  $Revision: 1.50 $
  $Id: gopdit.cpp,v 1.50 2006/10/06 07:53:18 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  mdify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include gopdit.cpp
*/

#include "config.h"
#include <sstream>
#include <sys/time.h>
#include <utime.h>
#include <vector>

#include "gopdit.h"
#include "editlist.h"
#include "mpeg2output.h"

using std::string;
using std::ostringstream;
using std::cerr;
using std::endl;
using std::flush;
using std::vector;

Parameter<string> filter_cmd(
    "FilterCommand",
//    "replex -k -i PS -t DVD -o \"%s\" &> /dev/null");
    "mencoder -of mpeg -mpegopts format=dvd:vbitrate=9000 -o \"%s\" "
    "-oac copy -ovc copy - &> /dev/null");

Parameter<string> vob_filter_cmd(
    "VobFilterCommand",
    "mencoder -of mpeg -mpegopts format=dvd:vbitrate=9000 -o \"%s\" "
    "-oac copy -ovc copy - &> /dev/null");

Parameter<bool> filter("ActivateFilter",false);

Parameter<bool> touch("Touch",false);

// super simple substitution function (some more consts??)
static string replacem(string s, vector<string>& mv, vector<string>& rv)
{
    assert( mv.size() == rv.size() ); 
    
    for ( unsigned i = 0; i < mv.size(); ++i ) 
    {
	string::size_type f = 0;
	while ( (f=s.find(mv[i],f)) != string::npos   ) 
	{
	    s = s.replace(f,mv[i].size(),rv[i]);
	    f++;
	}
    }
    return s;
}

void Gopdit::test(string where)
{
    if ( ui == 0 ) 
    {
	cerr << "ui is 0 in " << where << endl;
    }
    else
    {
	cerr << "ui seems ok in " << where << endl;
	ui->report("trying report");
    }
//    cerr << unsigned(ui) << endl;
    cerr << "pesmgrv size: "<< pesmgrv.size() << endl;
}

Gopdit::Gopdit()
    : pesmgr(0)
{
// insert error handling
    output = new GopditOutput();
//    output = 0;
}

Gopdit::~Gopdit()
{
    if ( pesmgr != 0 ) 
    {
	delete pesmgr;
    }
    if ( output != 0 ) 
    {
	delete output;
    }
}


bool Gopdit::openFile(string filename)
{
    // check, if file exists (open for testing)
    FILE* in;
    in = fopen(filename.c_str(),"r");
    if ( in == 0 ) 
    {
	ui->error(string("cannot open file: ") + filename);
	return false;
    }
    fclose(in);

    // extract moviename
    string moviename = 
	filename.substr(filename.rfind('/')+1, 
			filename.rfind('.')-filename.rfind('/')-1);

    pesmgr = new PesMgr(*ui, *output, filename.c_str(), moviename.c_str());
    pesmgrv.push_back(pesmgr);

    // do we have to play a bit of the movie for initialization?
    // getting type etc

    ui->report(string("opended file ") + filename);
    return true;
}


bool Gopdit::importFile(string filename, string importtype)
{
    // check, if file exists?
    Mpeg2ClipList tmp_clip_list;
    tmp_clip_list = GopditEditListMgr(*ui, *this).importList(
	filename.c_str(), importtype.c_str());

    ostringstream istr;
    istr << tmp_clip_list.size() << flush;
//    string str(stream.str());
//    return str;

    ui->report(string("found ") + istr.str() + " clips");

    for ( unsigned i = 0; i < tmp_clip_list.size(); ++i ) 
    {
	addClip(tmp_clip_list[i]);
    }
    ui->report(string("imported file ") + filename);
    return true;
}


bool Gopdit::saveFile(string filename, string savetype, 
		      Mpeg2ClipList& cut_list)
{

    FILE* out;
    bool   postprocessing = filter.getValue();
    string filtercmd      = filter_cmd.getValue();

    string filetype = string(savetype,0,savetype.find(" "));

    if ( filetype == "Vob" ) 
    {
	postprocessing = true;
	filtercmd      = vob_filter_cmd.getValue();
    }

    if ( postprocessing )
    {
	if ( filename == "-" ) 
	{
	    ui->error("cannot execute filter command writing ot stdout");
	    return false;
	}

	// substitute some formal parameters
	
	vector<string> mv;
	vector<string> rv;
	ostringstream tmpstream;

	mv.push_back("$FILE_OUT");
	rv.push_back(filename);

	mv.push_back("%s");
	rv.push_back(filename);

	tmpstream << total_clip_length();
	mv.push_back("$FILE_LENGTH");
	rv.push_back(tmpstream.str());
	
	tmpstream.clear();

	string cmd = replacem(filter_cmd.getValue(),mv,rv);

	out = popen(cmd.c_str(),"w");
	if ( out == 0 ) 
	{
	    ui->error("cannot execute filter command");
	    return false;
	}
	
	ui->report(cmd);
    }
    else 
    {
	if ( filename == "-" ) 
	{
	    out = stdout;
	}
	else
	{
	    out = fopen(filename.c_str(),"wb");
	    
	    if ( out == 0 ) 
	    {
		ui->error(string("cannot open file: ") + filename);
		return false;
	    }
	}
    }

    off_t saved_bytes = 0;
    unsigned no_frames = 0;
    bool  success = true;

    for ( unsigned i = 0; 
	  (i < cut_list.size()) 
	      && (success = cut_list[i].pesmgr->concat(
		      saved_bytes, out, 
		      cut_list[i].begin.off, cut_list[i].end.off,
		      no_frames, pesmgr->getFrameRate())); 
	  
	  ++i ) 
    {}

    if ( !success ) 
    {
	ui->error("error while saving\n");
    }

    if ( !cut_list.empty() ) 
    {
	clip_list[0].pesmgr->endOfProgram(out);
    }

    if ( postprocessing ) 
    {
	pclose(out);
    }
    else if ( out != stdout ) 
    {
	fclose(out);
    }

    if ( touch.getValue() && filename != "-" && clip_list.size() > 0 ) 
    {
	// compute time value (currently just take mtime of last clip)
	time_t mtm = clip_list.back().pesmgr->getMtime();
	mtm -= (pesmgr->getLastClipPoint().ms - clip_list.back().end.ms)/1000;

	struct utimbuf timbuf;
	timbuf.actime = mtm;
	timbuf.modtime = mtm;
	utime(filename.c_str(), &timbuf);
    }

    ui->report(string("saved to file ") + filename);
    return true;
}

bool Gopdit::saveFileSeparate(string filename, string savetype)
{
    Mpeg2ClipList cut_list;
    for ( unsigned i = 0; i < clip_list.size(); ++i ) 
    {
	cut_list.clear();
	cut_list.push_back(clip_list[i]);
	string filename_i = clip_list[i].filename;
	if ( filename_i == std::string() ) 
	{
	    ostringstream istr;
	    istr.fill('0');
	    istr.width(2);
	    istr << i << flush;
	    filename_i = filename + "." + istr.str() + ".mpg";
	}
	saveFile(filename_i, savetype, cut_list);
    }
    return true;
}


bool Gopdit::addClip(Mpeg2Clip& clip)
{
    if ( clip.end.off <= clip.begin.off ) 
    {
	ui->error(string("clip end precedes clip begin\n"));
	return false;
    }
    clip_list.push_back(clip);
    return true;
}


off_t Gopdit::total_clip_length()
{
    off_t total_length = 0;
    for ( unsigned i = 0; i < clip_list.size(); ++i ) 
    {
	total_length += clip_list[i].end.off - clip_list[i].begin.off;
    }
    return total_length;
}
