#ifndef GOPDIT_QT_H
#define GOPDIT_QT_H

/*! 
  \file gopdit_qt.h
  \brief main interface and operations

  \author R. Enders

  $Revision: 1.8 $
  $Id: gopdit_qt.h,v 1.8 2006/01/06 12:17:59 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html


  Sorry, the design is a mess: no separation between interface and operations


  \include gopdit_qt.h
*/

#include "config.h"
#include <string>
#include "gopdit.h"
#include ".ui/gopditbase.h"
#include "gopditui.h"

class QProgressDialog;
class HelpWindow;
class Fileinfo;
class Preferences;
class QFileDialog;

class GopditQt : public GopditBase
	       , public Gopdit
	       , public GopditUi
{
    Q_OBJECT

public:
    GopditQt( QApplication& papp, 
		QWidget* parent = 0, const char* name = 0,
		WFlags fl = WType_TopLevel );

    virtual ~GopditQt();

public slots:
// menu bar
void fileExit();
    void fileOpen();
    void fileClose();
    void fileSave();
    void fileSaveAs();
    void fileImport();
    void fileExport();
    void editCut();
    void editCopy();
    void editPaste();
    void zoomIn();
    void zoomOut();
    void zoomToggle();
    void helpAbout();
    void fileinfo();
    void helpContents();
// buttons etc
    void curMovieActivated(int idx);
    void movieSliderChanged(int);
    void movieSliderReleased();
    void mvPlayClicked();
    void mvStopClicked();
    void mvBwdClicked();
    void mvFwdClicked();
    void mvClipsDown();
    void mvClipsUp();
    void clipBeginClicked();
    void clipEndClicked();
    void clipCorrect();
    void helpDestroyed();
    void preferences();
    void filterToggle();
// clip table
    void clipsDoubleClicked(int row, int col, int button, const QPoint& mousePos);
// ui implementations
public:
    void  critical(std::string message);
    void  error(std::string message);
    void  fileOpen(std::string filename);
    void* getMovieScreen();
    void  processEvents();
    void  report(std::string message);
    void  updateClipSum();
    void  updateCurrentTime(unsigned t);
    bool  updateProgressDialog(off_t c);
    void  updateMovieSlider(off_t c);
    bool  warning(std::string message);
    virtual bool  addClip(Mpeg2Clip& clip);
private:
    std::string formatTime(unsigned ms) const;
    std::string formatOff(off_t off) const;
    off_t getFsize() const;
    off_t getCurGop() const;
    off_t getValidCurGop() const;
    unsigned getCurTime() const;
    void  refresh();
    void  stopPlaying();
    void  zoomInSteps(int steps);
    void  updateClip(int row, const Mpeg2Clip& clip);
    void  updateZoom();
    off_t movieSliderOffset();
    void  centerSlider(off_t pos);
    bool                modified;
    Fileinfo*           finfo;
    HelpWindow*         help;
    Preferences*        prefs;
    QString             filename;
    QString             filetype;
    QApplication&       app;
    QProgressDialog*    prog_diag;
    off_t               slider_begin;
    off_t               slider_end;
    QPixmap             pause;
    QPixmap             play;
};

#endif /* GOPDIT_QT_H */
