/*! 
  \file main.cpp
  \brief

  \author R. Enders

  $Revision: 1.22 $
  $Id: main.cpp,v 1.22 2006/01/06 12:17:59 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html
   
  \include main.cpp
*/


#include "config.h"
#include <signal.h>
#include <getopt.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include "parameter.h"
#include ".ui/gopditbase.h"
#include "pesmgr.h"
#include "gopdit_qt.h"
#include "gopdit_nox.h"
#include <vector>
#include <string>

using std::vector;
using std::string;

extern Parameter<bool> nox;
extern Parameter<bool> import_mpeg;
extern FileDialogParameter import_file;
Parameter<bool> separate("Separate", false, "when true, saves clips separately");

void myMessageOutput( QtMsgType type, const char *msg )
{
    switch ( type ) {
    case QtDebugMsg:
	fprintf( stderr, "Debug: %s\n", msg );
	break;
    case QtWarningMsg:
	fprintf( stderr, "Warning: %s\n", msg );
	break;
    case QtFatalMsg:
	fprintf( stderr, "Fatal: %s\n", msg );
//	abort();                    // deliberately core dump
    }
}


static void usage()
{
    std::cout << "gopdit [-h <options>] [<mpegin>]" 
	      << std::endl
	      << "\tgui mode" 
	      << std::endl;
    std::cout << "gopdit [-h <options>] -x [<mpegin>] <editlist> <mpegout>" 
	      << std::endl
	      << "\tcommand line mode" 
	      << std::endl
	      << "\t<editlist> and <mpegout> can be replaced by '-'" 
	      << std::endl
	      << "\tfor stdin and stdout repectively" 
	      << std::endl;
    std::cout << "-h" << std::endl
	      << "\tprints this help message" << std::endl;
    std::cout << "-s" << std::endl
	      << "\tsave clips separately" << std::endl;
    std::cout << "-t <editlist type>" << std::endl
	      << "\tdefines the editlist type to be imported" << std::endl;
    std::cout << "-x" << std::endl
	      << "\tinhibits firing up X" << std::endl;
    ParameterBase::writeEnvironment(std::cout);
}

void treat_options(int argc, char **argv)
{
    // v: with options otherwise without

    int longindex;
    vector<option>  longopts;

    option longopt;
    longopt.name = "help";
    longopt.has_arg = 0;
    longopt.flag = NULL;
    longopt.val  = 'h';
    longopts.push_back(longopt);

    /* construct parameter options */
    ParameterBase::genLongopts(longopts,1);

    longopt.name = 0;
    longopt.has_arg = 0;
    longopt.flag = NULL;
    longopt.val  = 0;
    longopts.push_back(longopt);

    char *optstring = "sht:x";

    while ( true ) 
    {
	int opt = getopt_long(argc, argv, optstring,
			      &longopts[0], &longindex);

	if ( opt == -1 ) 
	{
	    break;
	}

	switch ( opt ) 
	{
	case 'h':
	    usage();
	    exit (0);
	    break;
	case 's':
	    separate.setValue(true);
	    break;
	case 't':
	    import_file.sel_filter.setValue(optarg);
	    break;
	case 'x':
	    nox.setValue(true);
	    break;
	case 1:
	    ParameterBase::setValue(longopts[longindex].name,optarg);
	    break;
	default:
	    std::cout << "unknown option" << std::endl;
	}
    }
}


int main( int argc, char ** argv )
{
    std::string rcfile = getenv("HOME");
    rcfile += "/.gopditrc";

    ParameterBase::readResourceFile(rcfile);

    // handle options
    treat_options(argc, argv);


    if ( nox.getValue() ) 
    {
	if ( argc - optind < 2 || argc - optind > 3 ) 
	{
	    usage();
	    exit (1);
	}

	GopditNox gopdit;

	if ( argc - optind == 3 ) 
	{
	    return gopdit.remove_clips(string(argv[optind]), 
				       string(argv[optind+1]), 
				       string(argv[optind+2]))
		? 0
		: 1;
	}
	else
	{
	    import_mpeg.setValue(true);
	    return gopdit.remove_clips(string(""), 
				       string(argv[optind]), 
				       string(argv[optind+1]))
		? 0
		: 1;
	}

    }
    else 
    {

	// inhibit broken pipe signals
	signal(SIGPIPE, SIG_IGN);

//    qInstallMsgHandler( myMessageOutput );

	if ( argc - optind > 1 ) 
	{
	    usage();
	    exit (1);
	}

	QApplication app( argc, argv );
	
	GopditQt window(app);
	window.setCaption("Gopdit " VERSION);
	if ( argc - optind == 1 ) 
	{
	    window.fileOpen(string(argv[optind]));
	}

	window.show();
	app.connect( &app, SIGNAL( lastWindowClosed() ), 
		     &app, SLOT( quit() ) );
	
	return app.exec();
    }
}
