#ifndef GOPDIT_PARAMETER_H
#define GOPDIT_PARAMETER_H

// missing: initialization concept!!
// static stuff should be removed!!

/*! 
  \file parameter.h
  \brief parameter handling

  \author	Reinhard Enders

  $Revision: 1.11 $
  $Id: parameter.h,v 1.11 2006/01/06 12:17:59 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de
   
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html
   
  \include parameter.h
*/

#include <assert.h>
#include <string>
#include <iostream>
#include <map>
#include <vector>

using std::string;
using std::istream;
using std::ostream;
using std::map;

class ParameterBase;

struct option;

/*! 
  map between parameter names and parameter pointers
*/
class ParameterMap : public std::map<std::string, ParameterBase*>
{
public:
    ParameterMap();
    ~ParameterMap();
};


/*! 
  abstract parameter class
*/
class ParameterBase
{
    friend ostream& operator<<(ostream& out, const ParameterBase& p);

    friend istream& operator>>(istream& in, ParameterBase& p);

    ParameterBase();
public:
    //! constructor
    ParameterBase(const std::string name);
    //! constructor with description text
    ParameterBase(const std::string name, const std::string description);
    //! destructor
    virtual ~ParameterBase();
    //! read a value from input 'in'
    virtual void read(istream &in) = 0;
    //! write a value to output 'out'
    virtual void write(ostream &out) const = 0;
    //! access the parameter map
    static ParameterMap& getParameterMap();
    //! search the environment for parameters (prefixed by 'prefix')
    static void readEnvironment(const std::string& prefix);
    //! write all parameters as environment
    static void writeEnvironment(ostream& out, bool getopt=true);
    //! write resource file
    static void writeResourceFile(const std::string& resourcefile);
    //! read resource file
    static void readResourceFile(const std::string& resroucefile);
    //! set parameter value from string
    static void setValue(const string& name, const string& value);
    //! generate longopts vector (with val set to val)
    static void genLongopts(std::vector<option>& longopts, int val);
    //! get the description
    string getDescription() const { return mDescription; }
//protected:
public:
    const std::string mName;

    const std::string mDescription;

    std::string mValueString;
};

/*! 
  templated parameter
*/
template <class T>
class Parameter : public ParameterBase
{
public:
    //! constructor
    Parameter(const std::string name)
	: ParameterBase(name)
    {}

    //! constructor with initial value
    Parameter(const std::string name, const T value) 
	: ParameterBase(name)
	, mValue(value)
    {}
    //! constructor with initial value and description
    Parameter(const std::string name, const T value, 
	      const std::string description)
	: ParameterBase(name, description)
	, mValue(value)
    {}
    //! access the parameter value
    T    getValue() const { return mValue; }
    //! change the parameter value
    void setValue(T val) { mValue = val; }
    //! read the parameter value from input stream 'in'
    void read(istream& in) { in >> mValue; }
    //! write the parmaeter value to outpu stream 'out'
    void write(ostream& out) const { out << mValue; }

protected:
    T          mValue;
};


/* some compiler versions don't seem to like the declarations below
   - as they aren't used outside parameter.cpp in this application, 
   they can be commented away
*/
template<>
void Parameter<bool>::read(istream& in);

template<>
void Parameter<bool>::write(ostream& out) const;

// read until line end, don't stop at blanks!
template<>
void Parameter<string>::read(istream& in);

/**/

class FileDialogParameter
{
public:
    FileDialogParameter(string pn, string fs, string sf, string cp, string dir);
    Parameter<std::string>    filters;
    Parameter<std::string>    sel_filter;
    Parameter<std::string>    caption;
    Parameter<std::string>    dir;
};


#endif // GOPDIT_PARAMETER_H
