/*! 
  \file parfiledialog.cpp
  \brief file selection box with persistent parameters

  \author R. Enders

  $Revision: 1.4 $
  $Id: parfiledialog.cpp,v 1.4 2006/01/06 12:18:00 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  mdify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include parfiledialog.cpp
*/

#include "parfiledialog.h"
#include <qregexp.h>

using std::string;

ParFileDialog::ParFileDialog(FileDialogParameter& par, QWidget * parent, 
			 const char * name, bool modal, 
			 QFileDialog::Mode mode)
    : QFileDialog(parent, name, modal)
    , parameter(par)
{
    // initialize 
    setFilters( QString(parameter.filters.getValue().c_str()) );
    setSelectedFilter( QString(parameter.sel_filter.getValue().c_str()) );
    setCaption( QString(parameter.caption.getValue().c_str()) );
    setDir( QString(parameter.dir.getValue().c_str()) );
    setMode(mode);
}

/*
  if name is extended by one of the extensions of the filter, then it
  won't be changed. Otherwise, the first extension is appended to name
*/
QString ParFileDialog::selectedExtendedFile()
{
    QRegExp rx;
    QString name_ext("");
    QString name = selectedFile();
    QString filter = selectedFilter();

    // find a file extension of name
    rx.setPattern("(\\.\\w+)$");
    int pos = rx.search(name);
    if ( pos >= 0 ) 
    {
	name_ext = name.mid(pos);
    }
    
    // get all filter extensions
    rx.setPattern("\\*(\\.\\w+)[ ;$\\)]");
    rx.search(filter);
    QStringList list = rx.capturedTexts();
    QStringList::Iterator it = list.begin();
    pos = 0;
    while ( pos >= 0 )
    {
        pos = rx.search( filter, pos );
        if ( pos > -1 )
	{
            if ( rx.cap(1) == name_ext ) 
	    {
		return name;
	    }
            pos  += rx.matchedLength();
        }
    }
    pos = rx.search(filter, 0);
    if ( pos > -1 ) 
    {
	return QString(name) + rx.cap(1);
    }
    return name;
}

ParFileDialog::~ParFileDialog()
{
    // dirPath doesn't seem to work correctly with symbolic links
    const QDir* directory = dir();
//    parameter.dir.setValue(dirPath().ascii());
    parameter.dir.setValue(directory->absPath().ascii());
    parameter.sel_filter.setValue(selectedFilter().ascii());

//    using std::cerr;
//    using std::endl;
//    cerr << parameter.dir.mName << " dir: " << directory->absPath() << endl;
//    cerr << parameter.dir.mName << " path: " << dirPath() << endl;

    delete(directory);
}
