/*! 
  \file pesfilter.cpp
  \brief

  filter that extracts PES dynamically from MPEG2-PS

  \author R. Enders

  $Revision: 1.6 $
  $Id: pesfilter.cpp,v 1.6 2006/01/06 12:18:00 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include pesfilter.cpp
*/

#include "config.h"
#include "pesfilter.h"
#include "gopditui.h"

#undef FSEEK
#undef FTELL
#ifdef HAVE_FSEEKO
# define FSEEK      fseeko
# define FTELL      ftello
#else
# define FSEEK      fseek
# define FTELL      ftell
#endif

#include <assert.h>
#include <sys/stat.h>

PesFilter::PesFilter(GopditUi& pui, const char* pfilename)
    : filename(pfilename)
    , ui(pui)
{
    filename = pfilename;
    file     = fopen(pfilename,"rb");

    struct stat     file_stat;
    stat(pfilename,&file_stat);
    fsize    = file_stat.st_size;
    mtime    = file_stat.st_mtime;

    FSEEK(file,0,SEEK_SET);
    init_state();
}


PesFilter::~PesFilter()
{
    if ( file != 0 ) 
    {
	fclose(file);
    }
}


// mode search_only is a real quick ugly hack!!!!

bool   PesFilter::searchGop()
{
    return read(0,1,true) == 1;
}


int    PesFilter::seeko(off_t offset, int whence)
{
    int err = FSEEK(file, offset, whence);
    if ( err == 0 ) 
    {
	init_state();
    }
    return err;
}


off_t  PesFilter::tello()
{
    return FTELL(file);
}


bool   PesFilter::init_state()
{
    int in;
    // error, if there are not enough bytes left
    bunloaded = 0;
    skip      = true;
    video     = false;
    prev_ph   = off_t(-1);
    gop_ph    = off_t(-1);
    for ( unsigned i = 1; i < 4; ++i ) 
    {
	if ( (in = getc(file)) == EOF )
	{
	    return false;
	}
	buffer[i] = in;
    }
    return true;
}

void PesFilter::endOfProgram(FILE* out)
{
    uint8_t buffer[4];
    buffer[0] = 0;
    buffer[1] = 0;
    buffer[2] = 1;
    buffer[3] = 0xb9;
    fwrite(buffer,sizeof(uint8_t),4,out);
}
