/*! 
  \file tymgr.cpp
  \brief

  filter that extracts PES dynamically from MPEG2-PS

  \author R. Enders

  $Revision: 1.5 $
  $Id: tymgr.cpp,v 1.5 2006/01/06 12:18:00 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include tymgr.cpp
*/

#include "config.h"
#include "tymgr.h"
#include "gopditui.h"

#undef FSEEK
#undef FTELL
#ifdef HAVE_FSEEKO
# define FSEEK      fseeko
# define FTELL      ftello
#else
# define FSEEK      fseek
# define FTELL      ftell
#endif


#include <assert.h>

// first version 0: skip, 1: keep, 2: audio, 3: video
//               4: pack start 5: gop start
uint8_t ty_action[] =
{
    /* 00 - 0f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 10 - 1f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 20 - 2f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 30 - 3f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 40 - 4f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 50 - 5f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 60 - 6f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 70 - 7f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 80 - 8f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 90 - 9f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* a0 - af */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* b0 - bf */
    0, 0, 0, 1, 1, 1, 0, 1, 5, 1, 4, 0, 0, 0, 0, 0,
    /* c0 - cf */
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    /* d0 - df */
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    /* e0 - ef */
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
    /* f0 - ff */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
};


// mode search_only is a real quick ugly hack!!!!
size_t TyMgr::read(uint8_t* ptr, size_t nmemb, bool search_only)
{
    // absolutely primitive version
    // search for PES headers and print out the byte code
    // compare muxed and demuxed files

    // absolutely unoptimized
    // improvement: use buffer(s)


    int     in;
    int     hdl_cd = -1;    // search for header data length
    int     pd_cd  = -1;    // search for packet data
    size_t  bsize  = 0;     // so far output

    while ( bsize < nmemb && (in = getc(file)) != EOF ) 
    {
	if ( !skip && !search_only ) 
	{
	    *ptr = buffer[0];
	    ++bsize;
	    ++ptr;
	}

	// not efficient!!!
	buffer[0] = buffer[1];
	buffer[1] = buffer[2];
	buffer[2] = buffer[3];
	buffer[3] = in;

	// not efficient (use a small fsm)
	if ( buffer[2] == 1 && buffer[0] == 0 && buffer[1] == 0
	     // seems to be necessary, though I'm not sure, if
	     // mpeg streams are correct, if this is needed
	    && pd_cd < 0 && hdl_cd < 0 ) 
	{
	    switch ( ty_action[buffer[3]] ) 
	    {
	    case 0:
		skip = true;
		break;
	    case 1:
		skip = !video;
		break;
	    case 2:
		skip = true;
		video = false;
		break;
	    case 3:
		skip = true;
		video = true;
		// search for field header data length
		hdl_cd = 5;
		prev_ph = FTELL(file) - 4;
		break;
	    case 5:
		// similar to 1
		// gop marker found
		skip = !video;
		if ( !skip ) 
		{
		    gop_ph = prev_ph;
		}
		if ( search_only ) 
		{
		    return 1;
		}
		break;
	    default:
		skip = true;
		video = false;
		break;
	    }
	}
	else if ( hdl_cd == 0 ) 
	{
	    pd_cd = in + 4;
	}
	else if ( pd_cd == 0 ) 
	{
	    skip  = !video;
	}
	--hdl_cd;
	--pd_cd;
    }

    while ( !search_only && !skip && bsize < nmemb && bunloaded < 4 ) 
    {
	*ptr = buffer[bunloaded];
	++bunloaded;
	++bsize;
	++ptr;
    }

#ifdef GOPDIT_TRACING
    if ( hdl_cd >= 0 || pd_cd >= 0 ) 
    {
	fprintf(stderr,"incorrect length in filter at %llx\n",
		FTELL(file));
    }
#endif // GOPDIT_TRACING

    assert(hdl_cd < 0);
    assert(pd_cd < 0);

    return bsize;
}


bool   TyMgr::concat(off_t& sb, FILE* out, off_t from, off_t to,
		     unsigned&, float)
{
    FSEEK(file,from,SEEK_SET);

    bool start = true;

    int  counter = 0;

    const size_t buffer_size = 4096;
    size_t  size;
    uint8_t buffer[buffer_size];

    while ( from < to ) 
    {
	size = from + buffer_size <= to ? buffer_size : to - from;
	size = fread(buffer,sizeof(uint8_t),size,file);
	size = fwrite(buffer,sizeof(uint8_t),size,out);
	sb += size;
	from += size;
	if ( start == true ) 
	{
	    start = false;
/* don't know yet, what to check
	    assert( buffer[0] == 0
		    && buffer[1] == 0
		    && buffer[2] == 0x01
		    && buffer[3] == 0xba );
*/
	}

	counter = (counter +1) % 256;
	if ( counter == 0 ) 
	{
	   if ( !ui.updateProgressDialog(sb) ) 
	   {
	       return false;
	   } 
	}
    }
    return true;
}
